#include <stdlib.h>
#include <string.h>


#include <sys/confnet.h>
#include <arpa/inet.h>
#include <net/if_var.h>
#include <sys/socket.h>

#include "config.h"
#include "score.h"

#define MAILSERVER  "pop.gmx.de"
#define USERNAME    "mustermann@gmx.de"
#define PASSWORD    "secrect"


POP3ACCOUNT *account;
APPCONFIG acfg;

char trace_level;

static int hex2bin(char c)
{
    if(c >= '0' && c <= '9' )
        return c - '0';
    if(c >= 'a' && c <= 'f' )
        return c - 'a' + 10;
    return 0;
}

/*
 *
 */
int AddAccount(char *host, char *user, char *pass, u_short port, u_long spams)
{
    POP3ACCOUNT *pacc;
    char *cp;

    if((pacc = malloc(sizeof(POP3ACCOUNT))) != 0) {
        if((pacc->pacc_host = malloc(strlen(host) + 1)) == 0)
            free(pacc);
        else if((pacc->pacc_user = malloc(strlen(user) + 1)) == 0) {
            free(pacc->pacc_host);
            free(pacc);
        }
        else if((pacc->pacc_pass = malloc(strlen(pass) + 1)) == 0) {
            free(pacc->pacc_host);
            free(pacc->pacc_user);
            free(pacc);
        }
        else {
            strcpy(pacc->pacc_host, host);
            if((cp = strchr(pacc->pacc_host, ':')) != 0) {
                *cp++ = 0;
                pacc->pacc_port = atoi(cp);    
            }
            else
                pacc->pacc_port = port;
            strcpy(pacc->pacc_user, user);
            strcpy(pacc->pacc_pass, pass);
            pacc->pacc_count = 0;
            pacc->pacc_size = 0;
            pacc->pacc_mark = 0;
            pacc->pacc_spams = spams;
            pacc->pacc_next = account;
            account = pacc;
            return 0;
        }
    }
    return -1;
}

/*
 *
 */
int DelAccount(char *host, char *user)
{
    POP3ACCOUNT *lp;
    POP3ACCOUNT **lpp;

    lpp = &account;
    lp = account;

    while (lp) {
        if (strcmp(lp->pacc_host, host) == 0 && strcmp(lp->pacc_user, user) == 0) {
            *lpp = lp->pacc_next;
            free(lp->pacc_host);
            free(lp->pacc_user);
            free(lp->pacc_pass);
            free(lp);
            return 0;
        }
        lpp = (POP3ACCOUNT **)&lp->pacc_next;
        lp = lp->pacc_next;
    }
    return -1;
}

/*
 * List IP configuration.
 *
 * No change to retrieve the DNS' IP.
 */
int SetIp(CONST char *name, CONST char *addr)
{
    int rc = 0;
    u_long ip = inet_addr(addr);
    
    if(ip != (u_long)-1) {
        if (strcmp_P(name, PSTR("addr")) == 0)
            confnet.cdn_cip_addr = ip;
        else if (strcmp_P(name, PSTR("mask")) == 0) 
            confnet.cdn_ip_mask = ip;
        else if (strcmp_P(name, PSTR("gate")) == 0) 
            confnet.cdn_gateway = ip;
        else if (strcmp_P(name, PSTR("dns")) == 0) 
            acfg.acfg_dns = ip;
        else
            rc = -1;
    }
    else
        rc = -1;

    return rc;
}

int SetMac(CONST u_char *mac)
{
    int n = strlen(mac);
    int i;
    
    for (i = 0; i < n; i++) {
        if (mac[i] < '0' || (mac[i] > '9' && mac[i] < 'a') || mac[i] > 'f') {
            n = 13;
            break;
        }
    }
    if (n <= 12 && (n & 1) == 0) {
        n >>= 1;
        for (i = 0; i < n; i++)
            confnet.cdn_mac[6 - n + i] = hex2bin(mac[i * 2]) * 16 + hex2bin(mac[i * 2 + 1]);
        return 0;
    }
    return -1;
}

#define CONFAPP_EE_OFFSET   512

int ConfigSaveBinary(int addr, void *val, size_t len)
{
    size_t i;
    u_char *cp = val;

    for(i = 0; i < len; cp++, i++)
        if(eeprom_read_byte((void *) (addr + i)) != *cp)
            eeprom_write_byte((void *) (addr + i), *cp);

    return len;
}

int ConfigSaveString(int addr, u_char *str)
{
    int rc = 0;

    do {
        if(eeprom_read_byte((void *) (addr + rc)) != *str)
            eeprom_write_byte((void *) (addr + rc), *str);
        rc++;
    } while(*str++);

    return rc;
}

size_t ConfigLoadString(int addr, u_char *str, size_t size)
{
    size_t rc = 0;

    while(rc < size) {
        *str = eeprom_read_byte((void *) (addr + rc));
        rc++;
        if(*str++ == 0)
            break;
    }
    return rc;
}

int ConfigLoadBinary(int addr, void *val, size_t len)
{
    size_t i;
    u_char *cp = val;
    
    for(i = 0; i < len; cp++, i++) 
        *cp = eeprom_read_byte((void *) (addr + i));

    return len;
}

int ConfigSize(void)
{
    int rc = 0;
    POP3ACCOUNT *pacc;
    SCORELIST *list;

    list = list_to;
    while(list) {
        rc += strlen(list->scol_str) + 1;
        rc += sizeof(list->scol_val);
        list = list->scol_nxt;
    }
    rc++;

    list = list_from;
    while(list) {
        rc += strlen(list->scol_str) + 1;
        rc += sizeof(list->scol_val);
        list = list->scol_nxt;
    }
    rc++;

    list = list_subject;
    while(list) {
        rc += strlen(list->scol_str) + 1;
        rc += sizeof(list->scol_val);
        list = list->scol_nxt;
    }
    rc++;

    rc += strlen(acfg.acfg_secret) + 1;
    rc += sizeof(acfg.acfg_dns);

    pacc = account;
    while(pacc) {
        rc += strlen(pacc->pacc_host) + 1;
        rc += strlen(pacc->pacc_user) + 1;
        rc += strlen(pacc->pacc_pass) + 1;
        rc += sizeof(pacc->pacc_port);
        rc += sizeof(pacc->pacc_spams);
        pacc = pacc->pacc_next;
    }
    rc++;

    return rc;
}

int ConfigSave(void)
{
    int addr = CONFAPP_EE_OFFSET;
    POP3ACCOUNT *pacc;
    SCORELIST *list;

    NutNetSaveConfig();

    addr += ConfigSaveString(addr, "SPAMBIN");

    list = list_to;
    while(list) {
        addr += ConfigSaveString(addr, list->scol_str);
        addr += ConfigSaveBinary(addr, &list->scol_val, sizeof(list->scol_val));
        list = list->scol_nxt;
    }
    addr += ConfigSaveBinary(addr, 0, 1);

    list = list_from;
    while(list) {
        addr += ConfigSaveString(addr, list->scol_str);
        addr += ConfigSaveBinary(addr, &list->scol_val, sizeof(list->scol_val));
        list = list->scol_nxt;
    }
    addr += ConfigSaveBinary(addr, 0, 1);

    list = list_subject;
    while(list) {
        addr += ConfigSaveString(addr, list->scol_str);
        addr += ConfigSaveBinary(addr, &list->scol_val, sizeof(list->scol_val));
        list = list->scol_nxt;
    }
    addr += ConfigSaveBinary(addr, 0, 1);

    addr += ConfigSaveString(addr, dnsbl_domain);
    addr += ConfigSaveBinary(addr, &dnsbl_score, sizeof(dnsbl_score));

    addr += ConfigSaveString(addr, acfg.acfg_secret);
    addr += ConfigSaveBinary(addr, &acfg.acfg_dns, sizeof(acfg.acfg_dns));

    pacc = account;
    while(pacc) {
        addr += ConfigSaveString(addr, pacc->pacc_host);
        addr += ConfigSaveString(addr, pacc->pacc_user);
        addr += ConfigSaveString(addr, pacc->pacc_pass);
        addr += ConfigSaveBinary(addr, &pacc->pacc_port, sizeof(pacc->pacc_port));
        addr += ConfigSaveBinary(addr, &pacc->pacc_spams, sizeof(pacc->pacc_spams));
        pacc = pacc->pacc_next;
    }
    addr += ConfigSaveBinary(addr, 0, 1);

    return 0;
}

int ConfigLoad(FILE *uart)
{
    int addr = CONFAPP_EE_OFFSET;
    char *b1 = malloc(128);
    char *b2 = malloc(128);
    char *b3 = malloc(128);
    char bc;
    short bs;
    long bl;
    
    if(b1 == 0 || b2 == 0 || b3 == 0) {
        if(b1)
            free(b1);
        if(b2)
            free(b2);
        if(b3)
            free(b3);
        return -1;
    }
    
    addr += ConfigLoadString(addr, b1, 16);
    if(strcmp(b1, "SPAMBIN") == 0) {
        
        for(;;) {
            addr += ConfigLoadString(addr, b1, 128);
            if(*b1 <= 0)
                break;
            addr += ConfigLoadBinary(addr, &bc, sizeof(bc));
            AddScore(b1, bc, &list_to);
        }
        
        for(;;) {
            addr += ConfigLoadString(addr, b1, 128);
            if(*b1 <= 0)
                break;
            addr += ConfigLoadBinary(addr, &bc, sizeof(bc));
            AddScore(b1, bc, &list_from);
        }
        
        for(;;) {
            addr += ConfigLoadString(addr, b1, 128);
            if(*b1 <= 0)
                break;
            addr += ConfigLoadBinary(addr, &bc, sizeof(bc));
            AddScore(b1, bc, &list_subject);
        }
        
        
        addr += ConfigLoadString(addr, dnsbl_domain, sizeof(dnsbl_domain));
        addr += ConfigLoadBinary(addr, &dnsbl_score, sizeof(dnsbl_score));
        
        addr += ConfigLoadString(addr, acfg.acfg_secret, sizeof(acfg.acfg_secret));
        addr += ConfigLoadBinary(addr, &acfg.acfg_dns, sizeof(acfg.acfg_dns));
        
        for(;;) {
            addr += ConfigLoadString(addr, b1, 128);
            if(*b1 <= 0)
                break;
            addr += ConfigLoadString(addr, b2, 128);
            addr += ConfigLoadString(addr, b3, 128);
            addr += ConfigLoadBinary(addr, &bs, sizeof(bs));
            addr += ConfigLoadBinary(addr, &bl, sizeof(bl));
            AddAccount(b1, b2, b3, bs, bl);
        }
    }
    else {
        strcpy(acfg.acfg_secret, "nut");
        acfg.acfg_dns = 0;

        strcpy(dnsbl_domain, "relays.osirusoft.com");
        dnsbl_score = -1;
    
        AddAccount("pop.gmx.de", "mustermann@gmx.de", "secret", POP3PORT, 0);
        AddAccount("local.de", "mustermann", "secret", POP3PORT, 0);

        AddScore("loan", -1, &list_subject);
        AddScore("kornet.net", -1, &list_from);
        AddScore("hanmail.net", -1, &list_from);
        AddScore("hnaorsapladm", -1, &list_to);
    }
    free(b1);
    free(b2);
    free(b3);
    
    return 0;
}

