#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <arpa/inet.h>
#include <netdb.h>
#include <pro/dhcp.h>

#include "pop3c.h"

int NutPop3Command(POP3SESSION *pops, CONST char *fmt, ...)
{
    va_list ap;
    int rc = 0;

    va_start(ap, fmt);
    if(fmt && (rc = vfprintf(pops->pops_stream, fmt, ap)) >= 0)
        fflush(pops->pops_stream);
    va_end(ap);

    if(rc >= 0 && fgets(pops->pops_buff, sizeof(pops->pops_buff), pops->pops_stream))
        return strncmp(pops->pops_buff, "+OK", 3);
    return -1;
}

POP3SESSION *NutPop3Open(CONST char *host, CONST char *user, CONST char *pass, u_long tmo)
{
    POP3SESSION *pops;
    u_short port = 110;
    u_long rip;
    char *cp;

    if((pops = malloc(sizeof(POP3SESSION))) == 0)
        return 0;
    for(cp = pops->pops_buff; *host && *host != ':';) 
        *cp++ = *host++;
    *cp = 0;
    if(*cp == ':')
        port = atoi(cp + 1);
    cp = pops->pops_buff;

    if((rip = inet_addr(pops->pops_buff)) != (u_long)-1 || 
       (rip = NutDnsGetHostByName(pops->pops_buff)) != 0) {
        if((pops->pops_sock = NutTcpCreateSocket()) != 0) {
            if(NutTcpConnect(pops->pops_sock, rip, port) == 0) {
                NutTcpSetSockOpt(pops->pops_sock, SO_RCVTIMEO, &tmo, sizeof(tmo));
                if((pops->pops_stream = _fdopen((int)pops->pops_sock, "r+b")) != 0) {
                    if(NutPop3Command(pops, 0) == 0)
                        if(user == 0 || NutPop3Command(pops, "USER %s\r\n", user) == 0)
                            if(pass == 0 || NutPop3Command(pops, "PASS %s\r\n", pass) == 0)
                                return pops;
                    fclose(pops->pops_stream);
                }
            }
            NutTcpCloseSocket(pops->pops_sock);
        }
    }
    free(pops);
    return 0;
}

void NutPop3Close(POP3SESSION *pops)
{
    NutPop3Command(pops, "QUIT\r\n");
    fclose(pops->pops_stream);
    NutTcpCloseSocket(pops->pops_sock);
    free(pops);
}

char *NutPop3GetArg(POP3SESSION *pops, int num)
{
    char *cp = pops->pops_buff;

    while(*cp && num) {
        if(*cp == ' ') {
            num--;
            while(*cp == ' ')
                cp++;
        }
        else
            cp++;
    }
    return cp;
}

int NutPop3Status(POP3SESSION *pops, int num, u_long *sizep)
{
    if(num == 0) {
        if(NutPop3Command(pops, "STAT\r\n"))
            return -1;
    }
    else if(NutPop3Command(pops, "LIST %u\r\n", num))
        return -1;

    if(sizep)
        *sizep = atol(NutPop3GetArg(pops, 2));

    return atoi(NutPop3GetArg(pops, 1));
}

POP3HEADER *NutPop3Header(POP3HEADER *poph)
{
    if(poph) {
        if(poph->poph_to)
            free(poph->poph_to);
        if(poph->poph_from)
            free(poph->poph_from);
        if(poph->poph_reply_to)
            free(poph->poph_reply_to);
        if(poph->poph_return_path)
            free(poph->poph_return_path);
        if(poph->poph_subject)
            free(poph->poph_subject);
        if(poph->poph_date)
            free(poph->poph_date);
    }
    else
        poph = malloc(sizeof(POP3HEADER));
    if(poph)
        memset(poph, 0, sizeof(POP3HEADER));

    return poph;
}

static char *NutPop3AddHeadline_P(char **hline, PGM_P type, char *buff)
{
    size_t tlen = strlen_P(type);
    size_t hlen;

    if(strncmp_P(buff, type, tlen))
        return 0;

    if(*hline)
        return *hline;

    buff[MAX_HEADERLINELEN] = 0;
    hlen = strlen(buff) - tlen;
    if((*hline = malloc(hlen)) == 0)
        return buff + tlen + 1;

    strcpy(*hline, buff + tlen + 1);
    strlwr(*hline);

    return *hline;
}

static int NutPop3GetHeadlines(FILE *stream, POP3HEADER *poph, char *buff, size_t size)
{
    int rc = 0;
    char *cp;
    
    for(;;) {
        if(fgets(buff, size, stream) == 0)
            return -1;
        if((cp = strchr(buff, '\r')) != 0)
            *cp = 0;
        if((cp = strchr(buff, '\n')) != 0)
            *cp = 0;
        if(*buff == 0)
            break;
        if(*buff != '.') {
            rc++;
            if(strncmp_P(buff, PSTR("Received:"), 9) == 0) {
                char i = 0;
                u_char ipn[4];

                cp = buff;
                while(*cp) {
                    if(*cp == '.') {
                        if(i) {
                            if(++i > 4) 
                                i = -1;
                        }
                        else
                            i = -1;
                    }
                    else if(*cp >= '0' && *cp <= '9') {
                        if(i >= 0) {
                            if(i == 0)
                                i = 1;
                            ipn[i - 1] = atoi(cp);
                            while(*cp >= '0' && *cp <= '9')
                                cp++;
                            continue;
                        }
                    }
                    else {
                        if(i == 4) {
                            if(poph->poph_route_num < MAX_ROUTE_IP)
                                poph->popm_route_ip[poph->poph_route_num++] = *((u_long *)ipn);
                        }
                        i = 0;
                    }
                    cp++;
                }
                continue;
            }
            if(NutPop3AddHeadline_P(&poph->poph_to, PSTR("To:"), buff))
                continue;
            if(NutPop3AddHeadline_P(&poph->poph_from, PSTR("From:"), buff))
                continue;
            if(NutPop3AddHeadline_P(&poph->poph_reply_to, PSTR("Reply-To:"), buff))
                continue;
            if(NutPop3AddHeadline_P(&poph->poph_return_path, PSTR("Return-Path:"), buff))
                continue;
            if(NutPop3AddHeadline_P(&poph->poph_subject, PSTR("Subject:"), buff))
                continue;
            if(NutPop3AddHeadline_P(&poph->poph_date, PSTR("Date:"), buff))
                continue;
            rc--;
        }
        else if(*(buff + 1) == 0)
            break;
    }
    return rc;
}

int NutPop3GetLines(POP3SESSION *pops, char *buff, size_t size)
{
    int rc = 0;
    size_t cl;
    char *cp;

    while(strcmp(pops->pops_buff, ".")) {
        if(fgets(pops->pops_buff, sizeof(pops->pops_buff), pops->pops_stream) == 0) {
            rc = -1;
            break;
        }
        if((cp = strchr(pops->pops_buff, '\r')) != 0)
            *cp = 0;
        if((cp = strchr(pops->pops_buff, '\n')) != 0)
            *cp = 0;
        if(buff == 0 || size <= 2)
            continue;
        cp = pops->pops_buff;
        if(*cp == '.')
            cp++;
        if((cl = strlen(cp) + 1) >= size) {
            cl = size - 2;
            *(cp + cl) = 0;
        }
        if(cl) {
            strcpy(buff, cp);
            strlwr(buff);
            buff += cl;
            *buff++ = '\n';
            *buff = 0;
            size -= cl + 1;
            rc += cl + 1;
        }
    }
    return rc;
}

int NutPop3Retrieve(POP3SESSION *pops, int mno, POP3HEADER *poph, char *buff, size_t size)
{
    int rc = 0;

    *pops->pops_buff = 0;

    if(poph) {
        NutPop3Header(poph);
        if((buff == 0 || size == 0) && NutPop3Command(pops, "TOP %u 0\r\n", mno) == 0)
            rc = NutPop3GetHeadlines(pops->pops_stream, poph, pops->pops_buff, sizeof(pops->pops_buff));
        else if((rc = NutPop3Command(pops, "RETR %u\r\n", mno)) == 0)
            rc = NutPop3GetHeadlines(pops->pops_stream, poph, pops->pops_buff, sizeof(pops->pops_buff));
    }
    else
        rc = NutPop3Command(pops, "RETR %u\r\n", mno);
    if(rc >= 0) 
        rc = NutPop3GetLines(pops, buff, size);

    return rc;
}

