/*!
 * Copyright (C) 2003 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log$
 */

#include <stdlib.h>
#include <stdio.h>
#include <io.h>

#include <sys/version.h>
#include <sys/confnet.h>
#include <sys/heap.h>
#include <sys/timer.h>

#include <arpa/inet.h>
#include <netdb.h>
#include <pro/dhcp.h>

#ifdef ETHERNUT2
#include <dev/lanc111.h>
#else
#include <dev/nicrtl.h>
#endif
#include <dev/uartavr.h>
#include <dev/hd44780.h>
#include <dev/term.h>

#include "config.h"
#include "service.h"
#include "pop3c.h"
#include "score.h"

/*
 * List collected headers.
 */
static void ListHeader(FILE *stream, POP3HEADER *poph)
{
    u_char i;

    if(poph->poph_return_path)
        fprintf(stream, "Return-Path: %s\n", poph->poph_return_path);
    for(i = 0; i < poph->poph_route_num; i++)
        fprintf(stream, "Received: %s\n", inet_ntoa(poph->popm_route_ip[i]));
    if(poph->poph_from)
        fprintf(stream, "From: %s\n", poph->poph_from);
    if(poph->poph_to)
        fprintf(stream, "To: %s\n", poph->poph_to);
    if(poph->poph_subject)
        fprintf(stream, "Subject: %s\n", poph->poph_subject);
    if(poph->poph_reply_to)
        fprintf(stream, "Reply-To: %s\n", poph->poph_reply_to);
    if(poph->poph_date)
        fprintf(stream, "Date: %s\n", poph->poph_date);
}

/*
 * List collected headers.
 */
static int KillSpam(POP3ACCOUNT *pacc, FILE *uart, FILE *lcd)
{
    POP3SESSION *pops;
    POP3HEADER *poph;
    u_int num;
    u_int i;
    u_long total;
    
    /*
     * Connect mail server.
     */
    if(lcd)
        fprintf(lcd, "\nConnecting\n%.16s", pacc->pacc_host);
    if(trace_level)
        fprintf(uart, "Connecting %s...", pacc->pacc_host);
    if((pops = NutPop3Open(pacc->pacc_host, pacc->pacc_user, pacc->pacc_pass, 20000)) == 0) {
        if(lcd)
            fputs("\nfailed", lcd);
        if(trace_level)
            fputs("failed\n", uart);
        return -1;
    }

    /*
     * Retrieve mailbox status.
     */
    num = NutPop3Status(pops, 0, &total);
    if(lcd)
        fprintf(lcd, "\n%u mails/%lu bytes", num, total);
    if(trace_level)
        fprintf(uart, "OK\n%d mails %lu bytes\n", num, total);

    /*
     * Update watermark,
     */
    if(num != pacc->pacc_count || total != pacc->pacc_size) {
        pacc->pacc_count = num;
        pacc->pacc_size = total;
        if(num > 200) {
            if(pacc->pacc_mark > 200)
                pacc->pacc_mark = num - 200;
            else
                pacc->pacc_mark = 0;
        }
        else
            pacc->pacc_mark = 0;
    }

    if(pacc->pacc_mark < pacc->pacc_count) {

        /* Update watermark. */
        i = pacc->pacc_mark;
        num = pacc->pacc_mark + 20;
        if(num > pacc->pacc_count)
            num = pacc->pacc_count;

        /* Allocate header structure. */
        poph = NutPop3Header(0);

        /*
         * Loop for mails.
         */
        if(trace_level) {
            if(i < num)
                fprintf(uart, "Retrieving %u to %u\n", i + 1, num);
            else
                fputs("No new mails\n", uart);
        }
        for(; i < num; i++) {

            /* 
             * Retrieve mail headers. 
             */
            if(NutPop3Retrieve(pops, i + 1, poph, 0, 0) < 0) {
                if(trace_level)
                    fprintf(uart, "Reading %d failed\n", i + 1);
                if(lcd)
                    fprintf(lcd, ESC_POS "  %d failed", i + 1);
                break;
            }
            if(lcd)
                fprintf(lcd, ESC_POS "  %.16s", poph->poph_from);

            /*
             * Check mail score.
             */
            if(Score(poph) < 0) {
                if(NutPop3Command(pops, "DELE %d\r\n", i + 1)) {
                    if(trace_level)
                        fprintf(uart, "Deleting %d failed\n", i + 1);
                    if(lcd)
                        fprintf(lcd, ESC_POS "  %d failed", i + 1);
                }
                else
                    pacc->pacc_spams++;
                if(trace_level > 1) {
                    if(trace_level > 2) {
                        fprintf(uart, "--- %u SPAM ---\n", i + 1);
                        ListHeader(uart, poph);
                    }
                    else
                        fprintf(uart, "%4u SPAM: %s\n", i + 1, poph->poph_from);
                }
            }
            else {
                pacc->pacc_mark++;
                if(trace_level > 1) {
                    if(trace_level > 2) {
                        fprintf(uart, "+++ %u PASS +++\n", i + 1);
                        ListHeader(uart, poph);
                    }
                    else
                        fprintf(uart, "%4u PASS: %s\n", i + 1, poph->poph_from);
                }
            }
            if(lcd)
                fprintf(lcd, "\n" ESC_CLREOL "%lu Spams", pacc->pacc_spams);
        }

        /*
         * Release allocated memory.
         */
        NutPop3Header(poph);
        free(poph);
    }

    /*
     * Disconnect.
     */
    NutPop3Close(pops);
    if(lcd)
        fputs(ESC_CLRHOME, lcd);
    if(trace_level)
        fprintf(uart, "Disconnected, %u bytes free\n", NutHeapAvailable());

    return 0;
}

/*
 * Application entry.
 */
int main(void)
{
    u_long baud = 115200;
    FILE *lcd;
    FILE *uart;

    /*
     * Initialize the uart device.
     */
    NutRegisterDevice(&devUart0, 0, 0);
    uart = fopen("uart0", "r+");
    _ioctl(_fileno(uart), UART_SETSPEED, &baud);
    NutSleep(200);
    fprintf(uart, "\n\nNut/OS %s SpamBin 1.0\n", NutVersionString());

    /*
     * Load configuration.
     */
    ConfigLoad(uart);

    /*
     * Open LCD terminal device.
     */
    NutRegisterDevice(&devLcd, 0, 0);
    lcd = fopen("lcd", "w");

    /*
     * Send identify command and display Nut/OS version.
     */
    fprintf(lcd, ESC_CLRHOME ESC_CURSOROFF "Nut/OS %s", NutVersionString());
    fprintf(lcd, "\nSpamBin 1.0");
    NutSleep(500);

    /*
     * Register Realtek controller at address 8300 hex
     * and interrupt 5.
     */
    if(NutRegisterDevice(&devEth0, 0x8300, 5)) 
        fputs("Registering device eth0 failed\n", uart);

    /*
     * LAN configuration using EEPROM values or DHCP/ARP method.
     */
    fputs("\nDHCP ", lcd);
    if(NutDhcpIfConfig("eth0", 0, 60000)) {
        fputs("EEPROM/DHCP/ARP config failed\n", uart);
        ServiceStart(uart, 0);
    }
    else
        ServiceStart(uart, 1);

    for(;;) {
        POP3ACCOUNT *pacc;
        
        pacc = account;
        while(pacc) {
            KillSpam(pacc, uart, lcd);
            NutSleep(1000);
            pacc = pacc->pacc_next;
        }
        NutSleep(60000);
    }
}
